from django import forms
from .models import CustomUser

"""
Form untuk registrasi pengguna baru.
Menangani validasi input dan sanitasi data sebelum disimpan.
"""

class UserRegistrationForm(forms.ModelForm):
    """
    Form registrasi dengan validasi khusus untuk setiap field.
    Menggunakan CustomUser model sebagai dasar.
    """
    class Meta:
        model = CustomUser
        fields = [
            'nama',
            'password', 
            'tanggal_lahir', 
            'nomor_hp', 
            'email', 
            'url_blog', 
            'deskripsi_diri', 
            'kode_ladang', 
            'ukuran_ladang'
        ]
        widgets = {
            'password': forms.PasswordInput(),
            'tanggal_lahir': forms.DateInput(attrs={'type': 'date'}),
            'deskripsi_diri': forms.Textarea(attrs={'rows': 4}),
            'ukuran_ladang': forms.NumberInput(attrs={'min': 0, 'max': 20000})
        }
        labels = {
            'nama': 'Nama',
            'password': 'Kata Sandi',
            'tanggal_lahir': 'Tanggal Lahir',
            'nomor_hp': 'Nomor HP',
            'email': 'Email',
            'url_blog': 'URL Blog',
            'deskripsi_diri': 'Deskripsi Diri',
            'kode_ladang': 'Kode Ladang',
            'ukuran_ladang': 'Ukuran Ladang (m²)'
        }

    def clean(self):
        """
        Memastikan semua field terisi (tidak boleh kosong).
        Menampilkan pesan error dalam Bahasa Indonesia.
        """
        cleaned_data = super().clean()
        required_fields = self.fields.keys()
        for field in required_fields:
            if field not in cleaned_data or not cleaned_data[field]:
                label = self.fields[field].label or field.replace('_', ' ').title()
                raise forms.ValidationError(f"{label} tidak boleh kosong")
        return cleaned_data

    def clean_nomor_hp(self):
        """
        Membersihkan format nomor HP:
        - Menghapus karakter + dan -
        - Memastikan dimulai dengan 62
        - Memastikan panjang sesuai (8-15 digit)
        """
        nomor_hp = self.cleaned_data.get('nomor_hp')
        if nomor_hp:
            nomor_hp = nomor_hp.replace('+', '').replace('-', '')
            if not nomor_hp.startswith('62'):
                raise forms.ValidationError("Nomor HP harus diawali dengan '62'")
            return nomor_hp
        return nomor_hp