"""
Model untuk menyimpan data pengguna dengan validasi khusus.
Merupakan ekstensi dari AbstractUser Django dengan penghapusan field username.
"""

from django.contrib.auth.models import AbstractUser
from django.db import models
from django.core.exceptions import ValidationError
import re
from datetime import date
from dateutil.relativedelta import relativedelta

# Fungsi-fungsi validasi untuk setiap field
def validate_nama(value):
    """
    Memvalidasi nama pengguna.
    Hanya menerima huruf, angka, dan karakter khusus (., _, -).
    """
    if not re.match(r'^[a-zA-Z0-9._-]+$', value):
        raise ValidationError("Nama hanya boleh berisi huruf, angka, dan karakter (.,_,-)")

def validate_password(value):
    """
    Memvalidasi kata sandi.
    Harus memiliki minimal 8 karakter, mengandung huruf, angka, dan karakter spesial.
    """
    if not re.match(r'^(?=.*[A-Za-z])(?=.*\d)(?=.*[@$!%*#?&])[A-Za-z\d@$!%*#?&]{8,}$', value):
        raise ValidationError("Kata sandi harus minimal 8 karakter dan mengandung huruf, angka, dan karakter spesial")

def validate_tanggal_lahir(value):
    age = relativedelta(date.today(), value).years
    if age < 12:
        raise ValidationError("Usia minimal 12 tahun")

def validate_nomor_hp(value):
    value = value.replace('+', '').replace('-', '')
    if not re.match(r'^62[0-9]{6,13}$', value):
        raise ValidationError("Nomor HP harus dalam format: 62123456 (8-15 digit tanpa + atau -)")
    return value

def validate_deskripsi(value):
    if len(value) < 5 or len(value) > 1000:
        raise ValidationError("Deskripsi diri harus antara 5-1000 karakter")

def validate_kode_ladang(value):
    if not re.match(r'^L-[0-9]{5}$', value):
        raise ValidationError("Kode Ladang harus dalam format L-XXXXX (X adalah angka)")

def validate_ukuran_ladang(value):
    if not 0 <= value <= 20000:
        raise ValidationError("Ukuran ladang harus antara 0-20.000 m2")

class CustomUser(AbstractUser):
    USERNAME_FIELD = 'nama'
    REQUIRED_FIELDS = ['email']
    
    username = None  
    nama = models.CharField(
        max_length=255,
        validators=[validate_nama],
        unique=True  
    )
    password = models.CharField(
        max_length=255,
        validators=[validate_password]
    )
    tanggal_lahir = models.DateField(
        validators=[validate_tanggal_lahir]
    )
    nomor_hp = models.CharField(
        max_length=15,
        validators=[validate_nomor_hp]
    )
    email = models.EmailField(max_length=255, null=True)
    url_blog = models.URLField(max_length=255, null=True)
    deskripsi_diri = models.TextField(
        validators=[validate_deskripsi],
        null=True
    )
    kode_ladang = models.CharField(
        max_length=7,
        validators=[validate_kode_ladang],
        null=True
    )
    ukuran_ladang = models.IntegerField(
        validators=[validate_ukuran_ladang],
        null=True
    )